@@ -571,7 +571,9 @@ class CsvExporter:
         :param item:    single item in JSON format
         :return:        score percentage property or empty string if property does not exist
         """
-        if isinstance(get_json_property(item, "scoring", SCORE_PERCENTAGE), (float, int)):
+        if isinstance(
+            get_json_property(item, "scoring", SCORE_PERCENTAGE), (float, int)
+        ):
             return get_json_property(item, "scoring", SCORE_PERCENTAGE)
         elif isinstance(
             get_json_property(item, "scoring", COMBINED_SCORE_PERCENTAGE), (float, int)
